#from __future__ import annotations

import os
from sys import platform

if os.name != "nt" and platform != "darwin":
    from psutil import sensors_temperatures
from .Cpu_interface import Cpu_interface

CPU_VENDOR = "amd"


class Cpu_amd(Cpu_interface):
    def __init__(self, os, model):
        super().__init__(os, CPU_VENDOR, model)

    @Cpu_interface.model.setter
    def model(self, value: str):
        self.info = " ".join(value.split()[1:])
        self._model = " ".join([value.split()[2], value.split()[3]])

        if self._model.find("APU") != -1:
            self._model = f"{self._model.split('-')[0]} APU"


    def get_temp(self) -> float:
        if self.os == "windows":
            raise NotImplementedError(
                "Temperature report for AMD CPU's is not supported on Windows yet."
            )
        elif self.os == "macos":
            raise NotImplementedError(
                "Temperature report for AMD CPU's is not supported on MacOS yet."
            )
        elif (
            self.os == "linux" # and os.name != "nt" linux only
        ):  # os.name comparaison not needed, its just for the linter
            sensors = sensors_temperatures()
            if "k10temp" in sensors:
                temps = sensors["k10temp"]
                if len(temps) == 1:
                    return temps.current
                else:
                    for temp in temps:
                        if temp.label == "Tdie":
                            return temp.current

                raise Exception("No valid temperature value found.")

            raise Exception("No valid sensor found.")
        else:
            raise NotImplementedError("Unknown OS, no CPU temperature report.")



# from __future__ import annotations

from typing import Dict
import sys, os

from .run_rpc import Run_rpc
from .cycles import cycle0, cycle1, cycle2, cycle3, runmac, windows, pause
from .computer.Computer import Computer
from .args import parse_args
from .debugger import run_rpc_debug
from .update import update
from . import __init__ as __init__
from .resources import systemd_service


def main():
    args = parse_args()

    if args.update:
        update()
    if os.name != "nt" and sys.platform != "darwin":
        if args.install:
            systemd_service.install()
        if args.uninstall:
            systemd_service.uninstall()
        if args.enable:
            systemd_service.enable()
        if args.disable:
            systemd_service.disable()
        if args.start:
            systemd_service.start()
        if args.stop:
            systemd_service.stop()
        if args.status:
            systemd_service.status()
    if args.version:
        print("FetchCord version:", __init__.VERSION)
        sys.exit(0)
    if args.time:
        if int(args.time) < 15:
            print("ERROR: Invalid time set, must be > 15 seconds, cannot continue.")
            sys.exit(1)
        else:
            print("setting custom time %s seconds" % args.time)
    try:
        if args.help:
            sys.exit(0)
    except AttributeError:
        pass

    computer: Computer = Computer()

    if (
        not computer.neofetchwin
        and computer.host == "Host: N/A"
        and args.nodistro
        and args.noshell
        and args.nohardware
    ):
        print("ERROR: no hostline is available!")
        sys.exit(1)
    # printing info with debug switch
    if args.debug:
        run_rpc_debug(computer)

    run: Run_rpc = Run_rpc()

    if computer.neofetchwin:
        # wandowz
        loops: Dict = {}
        loops_indexes: Dict = {}

        if not args.nodistro:
            loops["windows"] = (computer.osinfoid, windows)
            loops_indexes[len(loops_indexes)] = "windows"
        if not args.nohardware:
            loops["cycle1"] = (computer.cpuid, cycle1)
            loops_indexes[len(loops_indexes)] = "cycle1"

        run.set_loop(
            loops,
            loops_indexes,
            computer.updateMap,
            int(args.poll_rate) if args.poll_rate else 3,
        )
        run.run_loop(computer)
    else:
        # loonix
        loops: Dict = {}
        loops_indexes: Dict = {}

        if not args.nodistro and computer.os != "macos":
            loops["cycle0"] = (computer.osinfoid, cycle0)
            loops_indexes[len(loops_indexes)] = "cycle0"
        if computer.os == "macos":
            loops["runmac"] = ("740822755376758944", runmac)
            loops_indexes[len(loops_indexes)] = "runmac"
        if not args.nohardware:
            loops["cycle1"] = (computer.cpuid, cycle1)
            loops_indexes[len(loops_indexes)] = "cycle1"
        if not args.noshell:
            loops["cycle2"] = (computer.terminalid, cycle2)
            loops_indexes[len(loops_indexes)] = "cycle2"
        if not args.nohost and computer.os != "macos":
            loops["cycle3"] = (computer.hostappid, cycle3)
            loops_indexes[len(loops_indexes)] = "cycle3"
        if args.pause_cycle:
            loops["pause"] = ("", pause)
            loops_indexes[len(loops_indexes)] = "pause"

        run.set_loop(
            loops,
            loops_indexes,
            computer.updateMap,
            int(args.poll_rate) if args.poll_rate else 3,
        )
        run.run_loop(computer)


if __name__ == "__main__":
    main()


# from __future__ import annotations

import logging
from sys import platform, exit
import sys
from typing import Callable, Dict, List, Tuple
import psutil, os

from ..run_command import exec_bash, run_command
from ..args import parse_args
from ..Logger import Logger
from .flatpak import enableFlatpak
from .resources import get_infos, get_default_config
from .cpu.get_cpu import get_cpu
from .cpu.Cpu_interface import Cpu_interface
from .gpu.get_gpu import get_gpu
from .gpu.Gpu_interface import GpuType, get_gpuid

args = parse_args()

logger = Logger(
    "fetchcord_computer.log",
    "fetchcord_computer",
    logging.DEBUG if args.debug else logging.INFO,
)


class Computer:
    parseMap: Dict[str, Callable]
    componentMap: Dict[str, List] = {}
    idsMap: Dict[str, Dict]

    os: str
    neofetchwin: bool = False
    neofetch: bool = False
    values: str
    laptop: bool = False
    uptime: float

    @property
    def memory(self) -> str:
        return self.get_component_line("Memory:")

    @property
    def osinfo(self) -> str:
        return self.get_component_line("OS:")

    @property
    def osinfoid(self) -> str:
        component = self.get_component_line("OS:")
        component = (component.split()[0] + component.split()[1]).lower()
        component_list = self.idsMap[self.idsMap["map"]["OS:"]]

        for comp, id in component_list.items():
            if component.lower().find(comp.lower()) >= 0:
                return id

        print(
            "Unknown {}, contact us on github to resolve this.".format(
                self.idsMap["map"]["OS:"]
            )
        )

        return component_list["unknown"]

    @property
    def motherboard(self) -> str:
        return self.get_component_line("Motherboard:")

    @property
    def motherboardid(self) -> str:
        return self.get_component_idkey("Motherboard:")

    @property
    def host(self) -> str:
        return self.get_component_line("Host:")

    @property
    def hostid(self) -> str:
        hostsplit = self.host.split()
        host_list: Dict[str, str] = self.idsMap[self.idsMap["map"]["Host:"]]

        for line in hostsplit:
            if line in host_list:
                return line

        # try to get MacBook hostid
        hostid = []
        hostjoin = " ".join(self.host)
        for numsplit in range(len(hostjoin)):
            if not hostjoin[numsplit].isdigit():
                hostid.append(hostjoin[numsplit])
        hostid = "".join(hostid)
        hostid = hostid.split()[1]

        if hostid in host_list:
            return host_list[hostid]
        else:
            return host_list["unknown"]

    @property
    def hostappid(self) -> str:
        return self.get_component_id("Host:")

    @property
    def cpu(self) -> str:
        key = "CPU:"
        cpus: List[Cpu_interface] = self.get_component(key)
        temp = []
        for cpu in cpus:
            temp.append(cpu.info)

        return "\n".join(temp) if len(cpus) > 0 else "{} N/A".format(key)

    @property
    def cpuid(self) -> str:
        temp: List[Cpu_interface] = self.get_component("CPU:")

        if len(temp) == 0:
            return self.idsMap[self.idsMap["map"]["CPU:"]]["unknown"]
        else:
            return temp[0].get_id(self.idsMap[self.idsMap["map"]["CPU:"]])

    @property
    def gpu(self) -> str:
        key = "GPU:"
        gpus: List[GpuType] = self.get_component(key)
        temp = []
        for gpu in gpus:
            if gpu.vendor == "amd" and self.os == "linux":
                temp.append(gpu.get_amdgpurender(gpus, self.laptop).rstrip().lstrip())
            else:
                temp.append(gpu.model.lstrip().rstrip())

        return "\n".join(temp) if len(gpus) > 0 else "{} N/A".format(key)

    @property
    def gpuid(self) -> str:
        return get_gpuid(
            self.idsMap[self.idsMap["map"]["GPU:"]], self.get_component("GPU:")
        )

    @property
    def disks(self) -> str:
        return self.get_component_line("Disk")

    @property
    def resolution(self) -> str:
        return self.get_component_line("Resolution:")

    @property
    def theme(self) -> str:
        return self.get_component_line("Theme:")

    @property
    def kernel(self) -> str:
        return self.get_component_line("Kernel:")

    @property
    def packages(self) -> str:
        return self.get_component_line("Packages:")

    @property
    def shell(self) -> str:
        return self.get_component_line("Shell:")

    @property
    def shellid(self) -> str:
        return self.get_component_id("Shell:")

    @property
    def terminal(self) -> str:
        return self.get_component_line("Terminal:")

    @property
    def terminalid(self) -> str:
        return self.get_component_id("Terminal:")

    @property
    def wm(self) -> str:
        return self.get_component_line("WM:")

    @property
    def wmid(self) -> str:
        return self.get_component_line("WM:").split()[0]

    @property
    def font(self) -> str:
        return self.get_component_line("Font:")

    @property
    def de(self) -> str:
        return self.get_component_line("DE:")

    @property
    def deid(self) -> str:
        value = self.get_component_line("DE:").split()[0]

        return "n/a" if value == "DE:" else value

    @property
    def dewmid(self) -> str:
        de = self.get_component_line("DE:")

        return "\n".join(
            ["" if de == "{} N/A".format("DE:") else de, self.get_component_line("WM:")]
        )

    @property
    def desktopid(self) -> str:
        deid = self.deid.lower()
        wmid = self.wmid.lower()

        if deid == "unity":
            if wmid == "compiz":
                return "unity"
            else:
                return wmid

        if deid != "n/a" and deid in self.idsMap[self.idsMap["map"]["DE:"]]:
            return deid
        elif deid == "n/a" and wmid in self.idsMap[self.idsMap["map"]["WM:"]]:
            return wmid
        else:
            print("Unknown DE/WM, contact us on github to resolve this.")
            return "unknown"

    @property
    def battery(self) -> str:
        if self.laptop:
            return self.get_component_line("Battery")
        else:
            return "{} N/A".format("Battery")

    @property
    def lapordesk(self) -> str:
        if self.laptop and self.os != "macos":
            return "laptop"
        else:
            return "desktop"

    @property
    def version(self) -> str:
        return os.popen("sw_vers -productVersion").read()

    @property
    def product(self) -> str:
        return os.popen("sysctl -n hw.model").read()

    @property
    def devicetype(self) -> str:
        if self.product[0:7] == "MacBook":
            return "laptop"
        else:
            return "desktop"

    @property
    def bigicon(self) -> str:
        try:
            return self.idsMap[self.idsMap["map"]["Version:"]][self.version[0:5]]
        except KeyError:
            print("Unsupported MacOS version")
            return "bigslurp"

    def __init__(self):
        super().__init__()

        self.parseMap = {
            "CPU:": get_cpu,
            "GPU:": get_gpu,
            "Disk": self.get_disk,
            "Memory:": self.get_memory,
            "OS:": self.get,
            "Motherboard:": self.get,
            "Host:": self.get,
            "Resolution:": self.get,
            "Theme:": self.get,
            "Kernel:": self.get,
            "Packages:": self.get,
            "Shell:": self.get,
            "Terminal:": self.get,
            "Font:": self.get,
            "DE:": self.get,
            "WM:": self.get,
            "Battery": self.get_battery,
        }

        self.idsMap = get_infos()
        self.uptime = psutil.boot_time()

        self.detect_os()
        self.detect_laptop()

        self.fetch_values()

    def fetch_values(self):
        self.neofetchwin, self.neofetch, self.values = self.detect_neofetch()

        self.neofetch_parser(self.values)

        if not bool(self.componentMap):
            args.config_path = ""
            args.noconfig = False

            self.neofetchwin, self.neofetch, self.values = self.detect_neofetch()
            self.neofetch_parser(self.values)

        terminallist = self.idsMap[self.idsMap["map"]["Terminal:"]]
        if args.terminal and args.terminal.lower() in terminallist:
            self.componentMap["Terminal:"] = [args.terminal.lower()]
        elif args.terminal and args.terminal.lower() not in terminallist:
            print(
                "\nInvalid terminal, only %s are supported.\n"
                "Please make a github issue if you would like to have your terminal added.\n"
                "https://github.com/MrPotatoBobx/FetchCord" % terminallist
            )
            sys.exit(1)

        if self.get_component("Font:", True) and args.termfont:
            print(
                "Custom terminal font not set because a terminal font already exists, %s"
                % self.font
            )
        elif not self.get_component("Font:", True) and args.termfont:
            self.componentMap["Font:"] = [args.termfont]

    def updateMap(self):
        """
        Clear the components values and fetch new ones
        """
        self.clearMap()
        self.neofetchwin, self.neofetch, self.values = self.detect_neofetch()
        self.neofetch_parser(self.values)

    def clearMap(self):
        """
        Clear the components values
        """
        for key in self.componentMap.keys():
            del self.componentMap[key][:]

    def neofetch_parser(self, values: str):
        if args.debug:
            print(values)
        lines = values.split("\n")
        for i in range(len(lines)):
            line = lines[i]
            for key, detectedFunction in [
                (key, value) for key, value in self.parseMap.items() if key in line
            ]:
                if key not in self.componentMap:
                    self.componentMap[key] = []
                detectedFunction(
                    self.os, self.componentMap[key], line.rstrip("\n"), key
                )

    def detect_os(self) -> str:
        if platform == "linux" or platform == "linux2":
            self.os = "linux"
        elif platform == "darwin":
            self.os = "macos"
        elif platform == "win32":
            self.os = "windows"
        else:
            raise Exception("Not a supported OS !")

        return self.os

    def detect_laptop(self) -> bool:
        if self.os != "linux":
            self.laptop = False
        else:
            for i in os.listdir("/sys/class/power_supply"):
                if i.startswith("BAT"):
                    self.laptop = True
                    break

        return self.laptop

    def detect_neofetch(self):
        neofetchwin = False
        neofetch = False
        values = None

        if self.os == "windows":
            try:
                values = run_command(["neofetch", "--noart"])
            except Exception:
                pass
            else:
                neofetchwin = True
        elif not neofetchwin:
            if self.os == "linux":
                enableFlatpak()

            default_config = get_default_config()

            try:
                if self.os == "windows":
                    values = run_command(
                        [
                            "neofetch",
                            "--config {}".format(
                                "none"
                                if args.noconfig
                                else (
                                    args.config_path
                                    if args.config_path
                                    else (default_config)
                                )
                            ),
                            "--stdout",
                        ],
                        shell=(self.os == "windows"),
                    )
                else:
                    values = exec_bash(
                        "neofetch --config {} --stdout".format(
                            "none"
                            if args.noconfig
                            else (
                                args.config_path
                                if args.config_path
                                else (default_config)
                            )
                        )
                    )
                if args.nfco:
                    with open(args.nfco) as f:
                        values = "\n".join(f.readlines())
                
            except Exception:
                print(
                    "ERROR: Neofetch not found, please install it or check installation and that neofetch is in PATH."
                )
                exit(1)
            else:
                neofetch = True

        return (neofetchwin, neofetch, values)

    def get_battery(self, os: str, line: List, value: str, key: str):
        """
        Append the Battery info from given neofetch line

        Parameters
        ----------
        value : str
            Neofetch extracted line
        """

        line.append(value[value.find(key) + len(key) + 2 :])

    def get_disk(self, os: str, line: List, value: str, key: str):
        """
        Append the Disk info from the given neofetch line to the Disk list

        Parameters
        ----------
        value : str
            Neofetch extracted line
        """

        line.append(value[value.find(key) + len(key) + 2 :])

    def get_memory(self, os: str, line: List, value: str, key: str):
        """
        Get the memory info from the given neofetch line

        Parameters
        ----------
        value : str
            Neofetch extracted line
        """

        if args.memtype == "gb":
            memgb = value.split()
            used = float(memgb[1].replace("MiB", ""))
            total = float(memgb[3].replace("MiB", ""))

            line.append(
                " ".join(
                    [
                        str(round(used / 1024, 2)),
                        "GiB /",
                        str(round(total / 1024, 2)),
                        "GiB",
                    ]
                )
            )
        else:
            line.append(value[value.find(key) + len(key) + 1 :])

    def get(self, os: str, line: List, value: str, key: str, valueOffset: int = 1):
        """
        Get the info from the given neofetch line

        Parameters
        ----------
        os: str
            Detected OS ("windows", "linux" or "macos")
        line: List
            List who will contains the values
        value : str
            Neofetch extracted line
        key : str
            Key for the dict
        valueOffset: int
            Offset for extracting the value without the key (default : 1)
        """

        line.append(value[value.find(key) + len(key) + valueOffset :])

    def get_component(self, key: str, quiet: bool = False):
        """
        Get component info from map

        Args:
            key (str): component key in map
        """
        try:
            return self.componentMap[key]
        except KeyError as err:
            if quiet:
                print("[KeyError]: {}".format(err), end="")

            return []

    def get_component_line(self, key: str) -> str:
        try:
            values = self.componentMap[key]
            return "\n".join(values) if len(values) > 0 else "{} N/A".format(key)
        except KeyError as err:
            print("[KeyError]: ", end="")
            print(err)

            return "{} N/A".format(key)

    def get_component_id(self, key: str) -> str:
        component = self.get_component_line(key).lower()
        component_list = self.idsMap[self.idsMap["map"][key]]

        for comp, id in component_list.items():
            if component.find(comp.lower()) >= 0:
                return id

        print(
            "Unknown {}, contact us on github to resolve this.".format(
                self.idsMap["map"][key]
            )
        )

        return component_list["unknown"]

    def get_component_idkey(self, key: str) -> str:
        component = self.get_component_line(key).lower()
        component_list = self.idsMap[self.idsMap["map"][key]]

        for comp, _ in component_list.items():
            if component.find(comp.lower()) >= 0:
                return comp

        print(
            "Unknown {}, contact us on github to resolve this.".format(
                self.idsMap["map"][key]
            )
        )

        return component_list["unknown"]



# from __future__ import annotations

import argparse


def parse_args():

    parser = argparse.ArgumentParser(
        description="Fetch Cord\n" "https://github.com/MrPotatoBobx/FetchCord"
    )
    parser.add_argument(
        "--nodistro", action="store_true", help="Don't show distro info."
    )
    parser.add_argument(
        "--nohardware", action="store_true", help="Don't show hardware info."
    )
    parser.add_argument(
        "--noshell", action="store_true", help="Don't show shell/terminal info."
    )
    parser.add_argument("--nohost", action="store_true", help="Don't show host info.")
    parser.add_argument(
        "--noconfig",
        action="store_true",
        help="Disable neofetch custom config. Enable if you have an incompatible custom configuration.",
    )
    parser.add_argument(
        "--time",
        "-t",
        metavar="TIME",
        action="store",
        help="Set custom time in seconds for cycles. Default is 30 seconds",
    )
    parser.add_argument(
        "--terminal",
        metavar="TERMINAL",
        action="store",
        help="Set custom Terminal (useful if using something like dmenu, or launching from a script).",
    )
    parser.add_argument(
        "--termfont",
        metavar="TERMFONT",
        action="store",
        help="Set custom Terminal Font (useful if neofetch can't get it).",
    )
    parser.add_argument(
        "--install",
        action="store_true",
        help="Install fetchcord as a systemd service (user) and enable it.",
    )
    parser.add_argument(
        "--uninstall",
        action="store_true",
        help="Uninstall fetchcord as a systemd service (user).",
    )
    parser.add_argument(
        "--enable",
        action="store_true",
        help="Enable fetchcord systemd service (user).",
    )
    parser.add_argument(
        "--disable",
        action="store_true",
        help="Disable fetchcord systemd service (user).",
    )
    parser.add_argument(
        "--start",
        action="store_true",
        help="Start fetchcord systemd service (user).",
    )
    parser.add_argument(
        "--stop",
        action="store_true",
        help="Stop fetchcord systemd service (user).",
    )
    parser.add_argument(
        "--status",
        action="store_true",
        help="Get fetchcord systemd service status (user).",
    )
    parser.add_argument(
        "--update",
        action="store_true",
        help="Update database of distros, hardware, etc.",
    )
    parser.add_argument(
        "--testing",
        action="store_true",
        help="Get files from testing branch instead of master.",
    )
    parser.add_argument("--debug", "-d", action="store_true", help="Enable debugging.")
    parser.add_argument(
        "--pause-cycle",
        "-p",
        action="store_true",
        help="Extra cycle that pauses for 30 seconds or custom time using --time argument.",
    )
    parser.add_argument(
        "--memtype",
        "-m",
        metavar="TYPE",
        action="store",
        help="Show Memory in GiB or MiB. Valid vaules are 'gb', 'mb'",
    )
    parser.add_argument(
        "--poll-rate",
        "-r",
        metavar="RATE",
        action="store",
        help="Set info polling rate.",
    )
    parser.add_argument(
        "--version", "-v", action="store_true", help="Print FetchCord Version."
    )
    parser.add_argument(
        "--config-path",
        "-c",
        action="store",
        help="Specify custom neofetch config path.",
    )
    parser.add_argument(
        "--fetchcord-config-path",
        "-fc",
        action="store",
        help="Specify custom fetchcord config path.",
    )
    parser.add_argument(
        "--nfco",
        "-nfco",
        action="store",
        help="nfco",
    )

    return parser.parse_args()



#from __future__ import annotations

import urllib.request, sys, os
from ..args import parse_args

args = parse_args()

def update():
    print("Updating database...")
    url = f"https://raw.githubusercontent.com/MrPotatoBobx/FetchCord/{'testing' if args.testing else 'master'}/fetch_cord/resources/fetchcord_ids.json"
    urllib.request.urlretrieve(
        url, os.path.dirname(__file__) + "/resources/fetchcord_ids.json"
    )
    sys.exit(0)



#from __future__ import annotations

from typing import List

from .Cpu_amd import Cpu_amd
from .Cpu_intel import Cpu_intel


def get_cpu(os: str, line: List, value: str, key: str):
    """
    Append the CPU info from the given neofetch line to the CPU list

    Parameters
    ----------
    os :
        OS type
    line : List
        Component line
    value : str
        Neofetch extracted line
    key : str
        Component key
    """

    vendor = value.replace(key, "").lstrip("").replace("Intel(R)", "Intel")

    if vendor.find("Intel") != -1 or vendor.find("Pentium") != -1:
        line.append(Cpu_intel(os, value))
    elif vendor.find("AMD") != -1:
        line.append(Cpu_amd(os, value))


#from __future__ import annotations

import time
from typing import Dict

from .computer.Computer import Computer
from .run_rpc import Run_rpc
from .args import parse_args

args = parse_args()


class Cycles:
    config: Dict[str, str]

    def __init__(self, config: Dict[str, str]):
        self.config = config


def pause(run: Run_rpc, key: str, computer: Computer):
    if args.debug:
        print("pause_cycle")
    if args.time:
        time.sleep(int(args.time))
    else:
        time.sleep(30)


def windows(run: Run_rpc, key: str, computer: Computer):
    if args.debug:
        print("w_cycle 0")

    run.try_update(
        key,
        state=computer.osinfo,
        details=computer.memory,
        large_image="big",
        large_text=computer.osinfo,
        small_image=computer.motherboardid,
        small_text=computer.motherboard,
        start=computer.uptime,
    )

    if args.time:
        time.sleep(int(args.time))
    elif args.nohardware:
        time.sleep(9999)
    else:
        time.sleep(30)
    run.try_clear(key)


def runmac(run: Run_rpc, key: str, computer: Computer):
    if args.debug:
        print("runmac")
        print("devicetype: %s" % computer.devicetype)
        print("product %s" % computer.product)
        print("bigicon: %s" % computer.bigicon)
        print("ver: %s" % computer.version)
        print("uptime: %s" % computer.uptime)

    run.try_update(
        key,
        state=computer.packages,  # update state as packages
        details=computer.kernel,  # update details as kernel
        large_image=computer.bigicon,  # set icon
        large_text=computer.osinfo,  # set large icon text
        small_image=computer.devicetype,  # set small image icon
        small_text=computer.product,  # set small image text
        start=computer.uptime,
    )
    if args.time:
        time.sleep(int(args.time))
    elif args.nohost and args.nohardware and args.noshell:
        time.sleep(9999)
    else:
        time.sleep(30)
    run.try_clear(key)


def cycle0(run: Run_rpc, key: str, computer: Computer):
    top_line = run.config["cycle_0"]["top_line"]
    if top_line == "kernel":
        top_line = computer.kernel
    else:
        top_line = computer.packages
    bottom_line = run.config["cycle_0"]["bottom_line"]
    if bottom_line == "kernel":
        bottom_line = computer.kernel
    else:
        bottom_line = computer.packages
    de_wm_icon = run.config["cycle_0"]["de_wm_icon"]
    if de_wm_icon == "on":
        de_wm_icon = computer.desktopid
    else:
        de_wm_icon = "off"
    if args.debug:
        print("cycle 0")

    run.try_update(
        key,
        state=bottom_line,
        details=top_line,
        large_image="big",
        large_text=computer.osinfo,
        small_image=de_wm_icon,
        small_text=computer.dewmid,
        start=computer.uptime,
    )
    if args.debug:
        print("appid: %s" % computer.osinfoid)
    config_time = run.config["cycle_0"]["time"]
    if args.time:
        time.sleep(int(args.time))
    elif args.nohost and args.nohardware and args.noshell:
        time.sleep(9999)
    elif config_time:
        time.sleep(int(config_time))
    else:
        time.sleep(30)
    run.try_clear(key)


def cycle1(run: Run_rpc, key: str, computer: Computer):
    top_line = run.config["cycle_1"]["top_line"]
    if top_line == "gpu":
        top_line = computer.gpu
    elif top_line == "cpu":
        top_line = computer.cpu
    elif top_line == "mem":
        top_line = computer.memory
    elif top_line == "disk":
        top_line = computer.disks
    bottom_line = run.config["cycle_1"]["bottom_line"]
    if bottom_line == "gpu":
        bottom_line = computer.gpu
    elif bottom_line == "cpu":
        bottom_line = computer.cpu
    elif bottom_line == "mem":
        bottom_line = computer.memory
    elif bottom_line == "disk":
        bottom_line = computer.disks
    gpu_icon = run.config["cycle_1"]["gpu_icon"]
    if gpu_icon == "on":
        gpu_icon = computer.gpuid
    else:
        gpu_icon = "off"
    if args.debug:
        print("cycle 1")
    run.try_update(
        key,
        state=bottom_line,
        details=top_line,
        large_image="big",
        large_text=computer.cpu,
        small_image=gpu_icon,
        small_text=computer.gpu,
        start=computer.uptime,
    )
    if args.debug:
        print("appid: %s" % computer.cpuid)
    config_time = run.config["cycle_1"]["time"]
    if args.time:
        time.sleep(int(args.time))
    elif args.nodistro and args.noshell and args.nohost:
        time.sleep(9999)
    elif config_time:
        time.sleep(int(config_time))
    else:
        time.sleep(30)
    run.try_clear(key)


def cycle2(run: Run_rpc, key: str, computer: Computer):
    top_line = run.config["cycle_2"]["top_line"]
    if top_line == "font":
        top_line = computer.terminal
    elif top_line == "shell":
        top_line = computer.shellid
    elif top_line == "theme":
        top_line = computer.theme
    bottom_line = run.config["cycle_2"]["bottom_line"]
    if bottom_line == "font":
        bottom_line = computer.terminal
    elif bottom_line == "shell":
        bottom_line = computer.shell
    elif bottom_line == "theme":
        bottom_line = computer.theme
    shell_icon = run.config["cycle_2"]["shell_icon"]
    if shell_icon == "on":
        shell_icon = computer.shellid
    else:
        shell_icon = "off"
    if args.debug:
        print("cycle 2")

    run.try_update(
        key,
        state=bottom_line,
        details=top_line,
        large_image="big",
        large_text=computer.terminal,
        small_image=shell_icon,
        small_text=computer.shell,
        start=computer.uptime,
    )
    if args.debug:
        print("appid: %s" % computer.osinfoid)

    config_time = run.config["cycle_2"]["time"]

    if args.time:
        time.sleep(int(args.time))
    elif args.nodistro and args.nohardware and args.nohost:
        time.sleep(9999)
    elif config_time:
        time.sleep(int(config_time))
    else:
        time.sleep(30)
    run.try_clear(key)


def cycle3(run: Run_rpc, key: str, computer: Computer):
    # if not then forget it
    if computer.host != "Host: N/A":
        top_line = run.config["cycle_3"]["top_line"]
        if top_line == "battery":
            top_line = computer.battery
        elif top_line == "host":
            top_line = computer.host
        elif top_line == "resolution":
            top_line = computer.resolution
        bottom_line = run.config["cycle_3"]["bottom_line"]
        if bottom_line == "resolution":
            bottom_line = computer.resolution
        elif bottom_line == "host":
            bottom_line = computer.host
        elif bottom_line == "battery":
            bottom_line = computer.battery
        lapordesk_icon = run.config["cycle_3"]["lapordesk_icon"]
        if lapordesk_icon == "on":
            lapordesk_icon = computer.lapordesk
        else:
            lapordesk_icon = "off"
        if args.debug:
            print("cycle 3")
        run.try_update(
            key,
            state=computer.resolution,
            details=computer.battery,
            large_image="big",
            large_text=computer.host,
            small_image=lapordesk_icon,
            small_text=computer.lapordesk,
            start=computer.uptime,
        )
        if args.debug:
            print("appid: %s" % computer.hostappid)
        config_time = run.config["cycle_3"]["time"]
        if args.time:
            time.sleep(int(args.time))
        elif args.nodistro and args.nohardware and args.noshell:
            time.sleep(9999)
        elif config_time:
            time.sleep(int(config_time))
        else:
            time.sleep(30)
    # back from whence you came
    run.try_clear(key)



#from __future__ import annotations

from fetch_cord.run_command import BashError, exec_bash
import os


def enableFlatpak():
    home = os.getenv("HOME")
    flatpak_discord_path = os.path.isdir("%s/.var/app/com.discordapp.Discord" % home)
    package_path = os.path.isfile("/usr/bin/discord")
    manual_install_path = os.path.isdir("/opt/Discord")
    if flatpak_discord_path and not package_path and not manual_install_path:
        XDG_Symlink(home)


def XDG_Symlink(home):
    try:
        print("Symlinking XDG_RUNTIME_DIR path for Flatpak Discord.")
        exec_bash(
            "cd %s/.var && ln -sf {app/com.discordapp.Discord,$XDG_RUNTIME_DIR}/discord-ipc-0 "
            % home
        )
    except BashError as e:
        print("Could not symlink XDG_RUNTIME_DIR Error: %s" % str(e))
        return



#from __future__ import annotations


from typing import Callable, Dict
from pypresence import Presence, exceptions
import time, sys

# import info about system
from .args import parse_args
from .config import ConfigError, load_config
from .computer.Computer import Computer

args = parse_args()


class Run_rpc:
    rpcs: Dict[str, Presence]
    config: Dict

    loops: Dict[str, Callable[['Run_rpc', str, Computer], None]] # Cannot use Run_rpc for type hinting unless doing the __future__.annotations import
    loops_indexes: Dict[int, str]
    poll_rate: int
    update: Callable

    def __init__(self):
        self.rpcs = {}

        try:
            self.config = load_config()
        except ConfigError as e:
            print("Error loading config file, using default values." % str(e))

    def set_loop(
        self, loops: Dict, loops_indexes: Dict, update: Callable, poll_rate: int = 3
    ):
        self.loops = loops
        self.loops_indexes = loops_indexes

        self.poll_rate = poll_rate
        self.update = update

    def run_loop(self, computer: Computer):
        try:
            loop = 0
            while True:
                for i in range(len(self.loops_indexes)):
                    if loop == self.poll_rate:
                        self.update()
                        loop = 0
                    try:
                        client_id, func = self.loops[self.loops_indexes[i]]

                        if args.debug:
                            print(self.rpcs)
                            print(
                                "{} not in : {}".format(
                                    self.loops_indexes[i],
                                    self.loops_indexes[i] not in self.rpcs,
                                )
                            )
                        if self.loops_indexes[i] not in self.rpcs:
                            self.rpcs[self.loops_indexes[i]] = Presence(client_id)
                        self.try_connect(self.loops_indexes[i])

                        func(self, self.loops_indexes[i], computer)
                        loop += 1
                    except ConnectionResetError:
                        self.try_connect(self.loops_indexes[i])
        except KeyboardInterrupt:
            print("Closing connection.")
            sys.exit(0)

    def try_connect(self, key: str):
        while True:
            try:
                if args.debug:
                    print('try_connect(key="{}") on {}'.format(key, self.rpcs[key]))
                self.rpcs[key].connect()
                break
            except ConnectionRefusedError:
                print(
                    "RPC connection refused (is Discord open?); trying again in 30 seconds"
                )
                time.sleep(30)

    def try_clear(self, key: str):
        # Pypresence clear doesn't work anymore
        # try:
        #     if args.debug:
        #         print(
        #             "[key={}] try_clear(pid={} on {}".format(
        #                 key, os.getpid(), self.rpcs[key]
        #             )
        #         )
        #     self.rpcs[key].clear(pid=os.getpid())
        # except exceptions.InvalidID:
        #     pass
        # except exceptions.ServerError as e:
        #     print(e)
        #     pass
        self.rpcs[key].close()

    def try_update(
        self,
        key: str,
        state,
        details,
        large_image,
        large_text,
        small_image,
        small_text,
        start,
    ):
        try:
            if args.debug:
                print('try_update(key="{}") on {}'.format(key, self.rpcs[key]))
            self.rpcs[key].update(
                state=state,
                details=details,
                large_image=large_image,
                large_text=large_text,
                small_image=small_image,
                small_text=small_text,
                start=start,
            )
        # ConnectionResetError is here to avoid crashing if Discord is still just starting
        except (ConnectionResetError, exceptions.InvalidID):
            pass


#from __future__ import annotations


try:
    import importlib.resources as pkg_resources
except ImportError:
    # Try backported to PY<37 `importlib_resources`.
    import importlib_resources as pkg_resources
import json

from .. import resources as fc_resources


def get_infos():
    with pkg_resources.open_text(fc_resources, "fetchcord_ids.json") as f:
        infos = json.load(f)

    return infos


def get_default_config():
    with pkg_resources.path(fc_resources, "default.conf") as path:
        return path

    return None



#from __future__ import annotations


try:
    import importlib.resources as pkg_resources
except ImportError:
    # Try backported to PY<37 `importlib_resources`.
    import importlib_resources as pkg_resources
import os, copy, json, configparser

from . import resources as fc_resources
from .args import parse_args


args = parse_args()


class ConfigError(Exception):
    pass


def load_config():
    default_config = configparser.ConfigParser()
    with pkg_resources.path(fc_resources, "fetch_cord.conf") as path:
        default_config.read_file(open(path))
        default_config.read(
            ["/etc/fetch_cord.conf", os.path.expanduser("~/fetch_cord.conf")]
        )
        if args.fetchcord_config_path != None:
            default_config.read([args.fetchcord_config_path])
        for item in default_config.items():
            print(item)

        default_config = _parsed_config_to_dict(default_config)
        _validate_config(default_config)

        return default_config


# def load_config():
#     base_config = configparser.ConfigParser()
#     base_config.read(["/etc/fetch_cord.conf"])
#     base_config = _parsed_config_to_dict(base_config)
#     _validate_config(base_config)


# #    try:
#     user_config = configparser.ConfigParser()
#     user_config.read("/etc/fetch_cord.conf")
#     user_config = _parsed_config_to_dict(user_config)
# #    except configparser.ParsingError as e:
# #        print(
# #            "Error parsing config file %s. Falling back to default config %s. Error is : %s",
# #            envs.USER_CONFIG_COPY_PATH, envs.DEFAULT_CONFIG_PATH, str(e))
#     return base_config

#     corrected_config = _validate_config(user_config, fallback_config=base_config)


#     return corrected_config


def _validate_config(config, fallback_config=None):

    folder_path = os.path.dirname(os.path.abspath(__file__))
    schema_path = os.path.join(folder_path, "config_schema.json")

    with open(schema_path, "r") as f:
        schema = json.load(f)

    corrected_config = copy.deepcopy(config)

    # Checking if the config file has the required sections and options
    for section in schema.keys():

        if section not in config.keys():
            raise ConfigError("Cannot find header for section [%s]" % section)

        for option in schema[section].keys():

            if option not in config[section].keys():
                raise ConfigError(
                    'Cannot find option "%s" in section [%s]' % (option, section)
                )

            valid, msg = _validate_option(
                schema[section][option], config[section][option]
            )

            if not valid:
                error_msg = (
                    'Config parsing : error in option "%s" in section [%s] : %s'
                    % (option, section, msg)
                )
                if fallback_config is not None:
                    print(error_msg)
                    print(
                        'Falling back to default value "%s"',
                        fallback_config[section][option],
                    )
                    corrected_config[section][option] = fallback_config[section][option]

                else:
                    raise ConfigError(error_msg)

    # Checking if the config file has no unknown section or option
    for section in config.keys():

        if section not in schema.keys():
            print("Config parsing : unknown section [%s]. Ignoring.", section)
            continue

        for option in config[section].keys():
            if option not in schema[section].keys():
                print(
                    'Config parsing : unknown option "%s" in section [%s]. Ignoring.',
                    option,
                    section,
                )
                del corrected_config[section][option]

    return corrected_config


def _parsed_config_to_dict(config):

    config_dict = {}

    for section in config.keys():

        if section == "DEFAULT":
            continue

        config_dict[section] = {}

        for option in config[section].keys():
            config_dict[section][option] = config[section][option]

    return config_dict


def _validate_option(schema_option_info, config_option_value):
    valid = False
    msg = "error"

    parameter_type = schema_option_info[0]

    assert parameter_type in ["multi_words", "single_word", "integer"]

    # Multiple-words parameters
    if parameter_type == "multi_words":
        valid, msg = _validate_multi_words(schema_option_info, config_option_value)

    # Single-word parameters
    elif parameter_type == "single_word":
        valid, msg = _validate_single_word(schema_option_info, config_option_value)

    # Integer parameter
    elif parameter_type == "integer":
        valid, msg = _validate_integer(schema_option_info, config_option_value)

    return valid, msg


def _validate_multi_words(schema_option_info, config_option_value):

    parameter_type, allowed_values, can_be_blank = schema_option_info
    assert parameter_type == "multi_words"

    values = config_option_value.replace(" ", "").split(",")

    if values == [""]:
        if not can_be_blank:
            msg = "at least one parameter required"
            return False, msg

    else:
        for val in values:
            if val not in allowed_values:
                msg = 'invalid value "%s"' % val
                return False, msg

    return True, None


def _validate_single_word(schema_option_info, config_option_value):

    parameter_type, allowed_values, can_be_blank = schema_option_info
    assert parameter_type == "single_word"

    val = config_option_value.replace(" ", "")

    if val == "":
        if not can_be_blank:
            msg = "non-blank value required"
            return False, msg

    else:
        if val not in allowed_values:
            msg = 'invalid value "%s"' % val
            return False, msg

    return True, None


def _validate_integer(schema_option_info, config_option_value):

    parameter_type, can_be_blank = schema_option_info
    assert parameter_type == "integer"

    val = config_option_value.replace(" ", "")

    if val == "":
        if not can_be_blank:
            msg = "non-blank integer value required"
            return False, msg

    else:
        try:
            v = int(val)
            if v <= 0:
                raise ValueError
        except ValueError:
            msg = "non-blank integer value required"
            return False, msg

    return True, None



#from __future__ import annotations

from .Cpu_interface import Cpu_interface
import re

CPU_VENDOR = "intel"


class Cpu_intel(Cpu_interface):
    def __init__(self, os, model):
        super().__init__(os, CPU_VENDOR, model)

    @Cpu_interface.model.setter
    def model(self, value: str):
        self.info = " ".join(value.split()[1:])
        if value.split()[1].replace("Intel(R)", "Intel") == "Pentium":
            self._model = value.split()[1]
        else:
            # Remove "CPU: ", "(R)" and "(TM)"
            self._model = " ".join(
                re.sub(r"\((.+)\)", "", value.replace("-", " ")).split()[1:]
            )

            # Core 2 Duo, Core 2 Quad
            if self._model.find("Intel Core") != -1:
                self._model = " ".join(self._model.split()[:4])
            else:
                self._model = " ".join(self._model.split()[:2])

            if self._model == "Intel Core":
                self._model = value.split()[1:5]
                self._model = " ".join(self._model)

    def get_temp(self):
        if self.os == "windows":
            raise NotImplementedError(
                "Temperature report for Intel CPU's is not supported on Windows yet."
            )
        elif self.os == "macos":
            raise NotImplementedError(
                "Temperature report for Intel CPU's is not supported on MacOS yet."
            )
        elif self.os == "linux":
            raise NotImplementedError(
                "Temperature report for Intel CPU's is not supported on Linux yet."
            )
        else:
            raise NotImplementedError("Unkown OS, no CPU temperature report.")



from __future__ import annotations
from fetch_cord.run_command import BashError, exec_bash
from .Gpu_interface import Gpu_interface

GPU_VENDOR = "nvidia"


class Gpu_nvidia(Gpu_interface):
    primeoffload: bool

    def __init__(self, os, model):
        super().__init__(os, GPU_VENDOR, model)

    @Gpu_interface.model.setter
    def model(self, value: str):
        self._model = value

    def get_temp(self):
        if self.os == "windows":
            raise NotImplementedError(
                "Temperature report for Nvidia GPU's is not supported on Windows yet."
            )
        elif self.os == "macos":
            raise NotImplementedError(
                "Temperature report for Nvidia GPU's is not supported on MacOS yet."
            )
        elif self.os == "linux":
            try:
                return exec_bash(
                    "nvidia-smi -q | awk '/GPU Current Temp/{print $5}' | sed 's/^/[/;s/$/°C]/'"
                )
            except BashError:
                pass
        else:
            raise NotImplementedError("Unknown OS, no GPU temperature report.")

    def check_primeoffload(self):
        # only show the GPU in use with optimus, show both if prime render offload
        self.primeoffload = False
        try:
            self.primeoffload = exec_bash('xrandr --listproviders | grep -o "NVIDIA-0"')
            return True
        except BashError:
            return False


#from __future__ import annotations

from abc import ABCMeta, abstractmethod
from typing import Dict

from ..Peripheral_interface import Peripherical_interface


class Cpu_interface(Peripherical_interface, metaclass=ABCMeta):
    vendor: str
    _model: str
    info: str

    @property
    def model(self) -> str:
        return self._model

    @model.setter
    @abstractmethod
    def model(self, value: str):
        raise NotImplementedError

    @property
    def temp(self) -> float:
        try:
            self._temp = self.get_temp()
        except NotImplementedError as e:
            try:
                raise e
            finally:
                e = None
                del e

        else:
            return self._temp

    @temp.setter
    def temp(self, value: float):
        self._temp = value

    def __init__(self, os, vendor, model):
        super().__init__(os)
        self.vendor = vendor
        self.model = model

    @abstractmethod
    def get_temp(self) -> float:
        raise NotImplementedError

    def get_id(self, cpu_list: Dict[str, str]) -> str:
        if self.model.lower() in cpu_list[self.vendor]:
            return cpu_list[self.vendor][self.model.lower()]
        else:
            return cpu_list["unknown"]



from __future__ import annotations
from abc import ABCMeta, abstractmethod
from typing import List, TypeVar, Dict

from ..Peripheral_interface import Peripherical_interface


class Gpu_interface(Peripherical_interface, metaclass=ABCMeta):
    vendor: str
    _model: str

    @property
    def model(self) -> str:
        return self._model

    @model.setter
    @abstractmethod
    def model(self, value: str):
        raise NotImplementedError

    @property
    def temp(self) -> float:
        try:
            self._temp = self.get_temp()
        except NotImplementedError as e:
            try:
                raise e
            finally:
                e = None
                del e

        else:
            return self._temp

    @temp.setter
    def temp(self, value: float):
        self._temp = value

    def __init__(self, os, vendor, model):
        super().__init__(os)
        self.vendor = vendor
        self.model = model

    @abstractmethod
    def get_temp(self) -> float:
        raise NotImplementedError


GpuType = TypeVar("GpuType", bound="Gpu_interface")


def get_gpuid(gpu_ids: Dict[str, str], gpus: List[GpuType]):
    vendors = []
    for i in range(len(gpus)):
        if gpus[i].model.split()[0] not in vendors:
            vendors.append(gpus[i].model.split()[0])

    gpuvendor = "".join(vendors).lower()

    if gpuvendor in gpu_ids:
        return gpu_ids[gpuvendor]
    else:
        print("Unknown GPU, contact us on github to resolve this.")
        return "unknown"



from .computer.Computer import Computer
from typing import Dict


class Cycle:
    name: str

    top_line: str = None
    bottom_line: str = None
    small_icon: str = None

    def __init__(self, name: str, config: Dict[str, str], computer: Computer):
        self.name = name

        if "top_line" in config["top_line"]:
            self.top_line = config["top_line"]
        if "bottom_line" in config["bottom_line"]:
            self.bottom_line = config["bottom_line"]
        if "small_icon" in config["small_icon"]:
            self.small_icon = config["small_icon"]


#from __future__ import annotations

from abc import ABCMeta


class Peripherical_interface(metaclass=ABCMeta):
    os: str

    def __init__(self, os):
        super().__init__()
        self.os = os


from __future__ import annotations
from typing import List

from .Gpu_amd import Gpu_amd
from .Gpu_nvidia import Gpu_nvidia
from .Gpu_intel import Gpu_intel


def get_gpu(os: str, line: List, value: str, key: str):
    """
    Append the GPU info from the given neofetch line to the GPU list

    Parameters
    ----------
    os :
        OS type
    line : List
        Component line
    value : str
        Neofetch extracted line
    key : str
        Component key
    """

    value = value.replace(key, "").lstrip("")
    splitValue = value.split()

    for v in splitValue:
        if v.upper() in ["AMD", "RADEON"]:
            line.append(Gpu_amd(os, value))
            return
        elif v.upper() in ["NVIDIA", "GEFORCE"]:
            line.append(Gpu_nvidia(os, value))
            return
        elif v.upper() in ["INTEL"]:
            line.append(Gpu_intel(os, value))
            return


#!/usr/bin/env python3
from os.path import dirname, join
import setuptools
import fetch_cord.__init__ as __init__

setuptools.setup(
    name="FetchCord",
    version=__init__.VERSION,
    description="grabs information about your Distro and displays it as Discord Rich Presence.",
    long_description=open(join(dirname(__file__), "README.md")).read(),
    long_description_content_type="text/markdown",
    url="https://github.com/MrPotatoBobx/FetchCord",
    author="MrPotatoBobx",
    author_email="junkahole23@protonmail.com",
    license="MIT",
    package_data={"fetch_cord": ["config_schema.json", "resources/default.conf", "resources/fetch_cord.conf", "resources/fetchcord_ids.json"]},
    packages=["fetch_cord"],
    include_package_data=True,
    install_requires=["pypresence", "psutil", "importlib-resources"],
    keywords=["distro", "info", "discord", "fetch"],
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    entry_points={
        "console_scripts": [
            "fetchcord=fetch_cord.__main__:main",
        ]
    },
)



#from __future__ import annotations

from typing import List
import subprocess


def run_command(command: List[str], shell: bool = False):
    return subprocess.run(
        command, encoding="utf-8", stdout=subprocess.PIPE, shell=shell
    ).stdout


class BashError(Exception):
    pass


def exec_bash(command: str):
    try:
        out = (
            subprocess.check_output(["bash", "-c", command], stderr=subprocess.STDOUT)
            .decode("utf8")
            .strip()
        )

    except subprocess.CalledProcessError as e:
        out = e.stdout.decode("utf8")
        raise BashError("Failed to execute '%s' :\n%s" % (command, out))
    except FileNotFoundError as e:
        raise BashError("BASH not installed on your computer...")

    return out



from __future__ import annotations
from .Gpu_interface import Gpu_interface

GPU_VENDOR = "intel"


class Gpu_intel(Gpu_interface):
    def __init__(self, os, model):
        super().__init__(os, GPU_VENDOR, model)

    @Gpu_interface.model.setter
    def model(self, value: str):
        self._model = value

    def get_temp(self):
        if self.os == "windows":
            raise NotImplementedError(
                "Temperature report for Intel GPU's is not supported on Windows yet."
            )
        elif self.os == "macos":
            raise NotImplementedError(
                "Temperature report for Intel GPU's is not supported on MacOS yet."
            )
        elif self.os == "linux":
            raise NotImplementedError(
                "Temperature report for Intel GPU's is not supported on Linux yet."
            )
        else:
            raise NotImplementedError("Unknown OS, no GPU temperature report.")



from __future__ import annotations
from fetch_cord.run_command import BashError, exec_bash
from typing import List
import sys

from .Gpu_interface import Gpu_interface, GpuType

GPU_VENDOR = "amd"


class Gpu_amd(Gpu_interface):
    def __init__(self, os, model):
        super().__init__(os, GPU_VENDOR, model)

    @Gpu_interface.model.setter
    def model(self, value: str):
        self._model = value

    def get_temp(self):
        if self.os == "windows":
            raise NotImplementedError(
                "Temperature report for AMD GPU's is not supported on Windows yet."
            )
        elif self.os == "macos":
            raise NotImplementedError(
                "Temperature report for AMD GPU's is not supported on MacOS yet."
            )
        elif self.os == "linux":
            raise NotImplementedError(
                "Temperature report for AMD GPU's is not supported on Linux yet."
            )
        else:
            raise NotImplementedError("Unknown OS, no GPU temperature report.")

    def get_amdgpurender(self, gpu_list: List[GpuType], laptop: bool) -> str:
        try:
            for i in range(len(gpu_list)):
                # assume DRI_PRIME=0 is the intel GPU
                if laptop and "intel" == gpu_list[i].vendor.lower():
                    i += 1
                if (
                    laptop
                    and "amd" == gpu_list[i].vendor.lower()
                    and gpu_list[i].model != self.model
                ):
                    i += 1

                env_prime = "DRI_PRIME=%s" % i
                return exec_bash(
                    "%s glxinfo | grep \"OpenGL renderer string:\" |sed 's/^.*: //;s/[(][^)]*[)]//g'"
                    % env_prime
                )
        except BashError as e:
            print("ERROR: Could not run glxinfo [%s]" % str(e))
            sys.exit(1)


#!/usr/bin/python3

from os import system
from ..run_command import BashError, exec_bash
from ..args import parse_args
import sys

args = parse_args()


def systemd_cmd(cmd: str):
    try:
        print(exec_bash(f"systemctl --user {cmd} --now fetchcord"))
    except BashError as err:
        print(err)
        sys.exit(1)


def install():
    try:
        exec_bash("mkdir -p ~/.local/share/systemd/user")
    except:
        print("Error : Cannot create directory...")
        sys.exit(1)

    try:
        exec_bash(
            f"wget -O ~/.local/share/systemd/user/fetchcord.service https://raw.githubusercontent.com/MrPotatoBobx/FetchCord/{'testing' if args.testing else 'master'}/systemd/fetchcord.service",
        )
    except:
        print("Error: Failed to download the service file.")
        sys.exit(1)

    systemd_cmd("enable")

    start()


def uninstall():
    systemd_cmd("stop")

    systemd_cmd("disable")

    try:
        exec_bash("rm -f ~/.local/share/systemd/user/fetchcord.service")
    except:
        print("Error : Cannot remove service file...")
        sys.exit(1)

    sys.exit(0)


def enable():
    systemd_cmd("enable")
    sys.exit(0)


def disable():
    systemd_cmd("disable")
    sys.exit(0)


def start():
    systemd_cmd("start")
    sys.exit(0)


def stop():
    systemd_cmd("stop")
    sys.exit(0)


def status():
    systemd_cmd("status")
    sys.exit(0)


#from __future__ import annotations

from typing import List

from .computer.Computer import Computer
from .computer.cpu.Cpu_interface import Cpu_interface


def run_debug(computer: Computer):
    print("----out.py----\n")
    print("----DE/WM----")
    if computer.os != "windows":
        print("deid: %s" % computer.deid)
        print("wmid: %s" % computer.wmid)
        try:
            print("wmline item 0: %s" % computer.wm)
        except IndexError:
            pass
        print("\n----TERMINAL----\n")
        print("fontline: %s" % computer.font)
        print("termid: %s" % computer.terminalid)
        print("termline item 0: %s" % computer.terminal)
        print("themeline: %s" % computer.theme)
        if computer.host != "Host: N/A":
            print("\n----HOST INFO----\n")
            print("hostline: %s" % computer.host)
            if computer.battery != computer.host:
                print("batteryline: %s" % computer.battery)
            print("resline: %s" % computer.resolution)
    print("\n----GPU INFO----\n")
    print("gpuinfo: %s" % computer.gpu)
    print("gpuvendor: %s" % computer.gpuid)
    print("\n----CPU INFO----\n")
    cpu: List[Cpu_interface] = computer.get_component("CPU:")
    if cpu:
        print("cpuvendor: %s" % cpu[0].vendor)
        print("cpumodel: %s" % cpu[0].model)
        print("cpuinfo: %s" % cpu[0].info)
    print("cpuline item 0: %s" % computer.cpu)
    print("memline: %s" % computer.memory)
    print("\n----OS INFO----\n")
    print("sysosline: %s" % computer.osinfo)
    print("sysosid: %s" % computer.osinfoid)
    print("diskline: %s" % computer.disks)
    if computer.os != "windows":
        print("packagesline item 0: %s" % computer.packages)


def test_debug(computer: Computer):
    print("\n----testing.py----")
    if computer.os != "windows":
        print("----DE/WM----\n")
        print("deid: %s" % computer.deid)
        print("wmid: %s" % computer.wmid)
        print("\n----TERMINAL/SHELL----\n")
        print("termid: %s" % computer.terminalid)
        print("shellid: %s" % computer.shellid)
        print("\n----HOST INFO----\n")
        print("hostid: %s" % computer.hostid)
    else:
        print("moboid: %s" % computer.motherboardid)
        print("moboline: %s" % computer.motherboard)
    print("\n----GPU INFO----\n")
    print("gpuvendor: %s" % computer.gpuid)
    print("\n----CPU INFO----\n")
    cpu = computer.get_component("CPU:")
    if cpu:
        print("cpumodel: %s\n" % cpu[0].model)


def run_rpc_debug(computer: Computer):
    print("----run_rpc----\n")
    print("uptime in epoch: %s" % computer.uptime)
    print("cpuid: %s" % computer.osinfoid)
    print("cpuappid: %s" % computer.cpuid)
    if computer.os != "windows":
        print("termappid: %s" % computer.terminalid)
        if computer.host != "Host: N/A":
            print("hostappid: %s" % computer.hostappid)
        print(computer.packages)

    run_debug(computer)
    test_debug(computer)


import sys
import logging
from logging.handlers import TimedRotatingFileHandler

class Logger(logging.Logger):
    FORMATTER = logging.Formatter(
        "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    )

    file: str
    console_handler: logging.StreamHandler
    file_handler: TimedRotatingFileHandler

    def get_console_handler() -> logging.StreamHandler:
        console_handler = logging.StreamHandler(sys.stdout)
        console_handler.setFormatter(Logger.FORMATTER)

        return console_handler

    def get_file_handler(file: str) -> TimedRotatingFileHandler:
        file_handler = TimedRotatingFileHandler(file, when="midnight")
        file_handler.setFormatter(Logger.FORMATTER)

        return file_handler

    def __init__(self, file: str, name: str, level: int = logging.INFO):
        super().__init__(logging.getLogger(name))

        self.file = file
        self.console_handler = Logger.get_console_handler()
        self.file_handler = Logger.get_file_handler(file)

        self.setLevel(level)
        self.addHandler(self.console_handler)
        self.addHandler(self.file_handler)

        self.propagate = False